from dartrs.dartrs import DartTokenizer

TOKENIZER_NAME = "p1atdev/dart-v2-moe-sft"


def test_load_tokenizer():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)
    assert tokenizer is not None


def test_encode_text():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    text = "1girl, cat ears"
    encoded = tokenizer.encode(text)

    assert encoded is not None
    assert len(encoded) > 0
    assert encoded == [103, 255]


def test_decode_text():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode(encoded)

    assert decoded is not None
    assert decoded == "1girl, cat ears"


def test_decode_tags():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode_tags(encoded)

    assert decoded is not None
    assert decoded == ["1girl", "cat ears"]


def test_decode_tags_text_with_special_tokens():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode_tags(encoded, skip_special_tokens=False)

    assert decoded is not None
    assert decoded == ["1girl", "cat ears", "</general>", "<|eos|>"]


def test_tokenize_text():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    text = "1girl, cat ears, hogeeee"
    tokens = tokenizer.tokenize(text)

    assert tokens is not None
    assert tokens == ["1girl", "cat ears", "<|unk|>"]


def test_get_vocab():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    vocab = tokenizer.get_vocab(True)
    assert vocab is not None
    assert "<|unk|>" in vocab

    vocab_without_added_tokens = tokenizer.get_vocab(False)
    assert vocab_without_added_tokens is not None

    assert "<|unk|>" not in vocab_without_added_tokens
    assert len(vocab) > len(vocab_without_added_tokens)


def test_get_added_tokens():
    tokenizer = DartTokenizer.from_pretrained(TOKENIZER_NAME)

    added_tokens = tokenizer.get_added_tokens()
    assert added_tokens is not None
    assert len(added_tokens) > 0



from dartrs.dartrs import (
    DartV2Mistral,
    DartV2Mixtral,
    DartDevice,
    DartTokenizer,
    GenerationConfig,
)
from dotenv import load_dotenv
import os

load_dotenv()


def test_device():
    device = DartDevice.Cpu()
    assert device is not None

    device = DartDevice.Cuda(0)
    assert device is not None


def test_tokenizer():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-moe-sft")

    assert tokenizer is not None


def test_generation_config():
    config = GenerationConfig(
        device=DartDevice.Cpu(),
        tokenizer=DartTokenizer.from_pretrained("p1atdev/dart-v2-moe-sft"),
        prompt="<|bos|><copyright></copyright><character></character><|rating:sfw|><|aspect_ratio:tall|><|length:long|><general>1girl<|identity:lax|><|input_end|>",
    )

    assert config is not None


def test_mistral_model():
    model_name = "p1atdev/dart-v2-sft"

    model = DartV2Mistral(model_name)
    assert model is not None


def test_mixtral_model():
    model_name = "p1atdev/dart-v2-moe-sft"

    model = DartV2Mixtral(model_name)
    assert model is not None


def test_load_model_with_auth_token():
    TEST_HF_TOKEN = os.getenv("TEST_HF_TOKEN")

    model_name = "p1atdev/dart-v2-moe-sft"
    model = DartV2Mixtral(model_name, auth_token=TEST_HF_TOKEN)

    assert model is not None



from dartrs import dartrs, v2


def test_compose_prompt_v2():
    prompt = dartrs.compose_prompt_v2(
        copyright="vocaloid",
        character="hatsune miku",
        rating=dartrs.RatingTag.Sfw,
        aspect_ratio=dartrs.AspectRatioTag.Tall,
        length=dartrs.LengthTag.Long,
        identity_level=dartrs.IdentityTag.Lax,
        prompt="1girl, cat ears",
        do_completion=True,
    )

    assert prompt is not None
    assert prompt == (
        f"<|bos|>"
        f"<copyright>vocaloid</copyright>"
        f"<character>hatsune miku</character>"
        f"<|rating:sfw|><|aspect_ratio:tall|><|length:long|>"
        f"<general>1girl, cat ears<|identity:lax|><|input_end|>"
    )


def test_compose_prompt_v2_wrapper():
    prompt = v2.compose_prompt(
        prompt="1girl, cat ears",
        copyright="vocaloid",
        character="hatsune miku",
        rating="sfw",
        aspect_ratio="tall",
        length="long",
        identity="lax",
        do_completion=True,
    )

    assert prompt is not None
    assert prompt == (
        f"<|bos|>"
        f"<copyright>vocaloid</copyright>"
        f"<character>hatsune miku</character>"
        f"<|rating:sfw|><|aspect_ratio:tall|><|length:long|>"
        f"<general>1girl, cat ears<|identity:lax|><|input_end|>"
    )



from dartrs.v2 import MistralModel, MixtralModel
from dotenv import load_dotenv
import os

load_dotenv()


def test_v2_mistral_model():
    model = MistralModel.from_pretrained("p1atdev/dart-v2-sft")

    assert model is not None


def test_v2_mixtral_model():
    model = MixtralModel.from_pretrained("p1atdev/dart-v2-moe-sft")

    assert model is not None


def test_v2_mistral_model_with_auth_token():
    TEST_HF_TOKEN = os.getenv("TEST_HF_TOKEN")

    model = MistralModel.from_pretrained(
        "p1atdev/dart-v2-sft", auth_token=TEST_HF_TOKEN
    )

    assert model is not None



from dartrs.dartrs import DartTokenizer, GenerationCache
from dartrs.v2 import (
    MixtralModel,
    compose_prompt,
)
from dartrs.utils import get_generation_config, DType, Device
from random import randint


def prepare_models(dtype: DType = "fp32", device: Device = "cpu"):
    model = MixtralModel.from_pretrained("p1atdev/dart-v2-moe-sft", dtype=dtype)
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-moe-sft")

    return model, tokenizer


def test_generate():
    model, tokenizer = prepare_models()

    prompt = compose_prompt(
        prompt="1girl, cat ears",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=42,
    )

    output = model.generate(config)
    assert output is not None


def test_generate_fp16_cpu():
    model, tokenizer = prepare_models(dtype="fp16", device="cpu")

    prompt = compose_prompt(
        prompt="1girl, cat ears",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=42,
    )

    output = model.generate(config)
    assert output is not None


def test_generate_fp16_cuda():
    model, tokenizer = prepare_models(dtype="fp16", device="cuda")

    prompt = compose_prompt(
        prompt="1girl, cat ears",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=42,
    )

    output = model.generate(config)
    assert output is not None


def test_generate_different_seed():
    model, tokenizer = prepare_models()

    seeds = [1, 2900]
    results = []

    for seed in seeds:
        prompt = compose_prompt(
            prompt="1girl, cat ears",
        )
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=seed,
        )

        results.append(model.generate(config))

    assert results[0] != results[1]


def test_generate_random_seed():
    model, tokenizer = prepare_models()

    seeds = [randint(0, 10000) for _ in range(10)]
    results = []

    for seed in seeds:
        prompt = compose_prompt(
            prompt="1girl, cat ears",
        )
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=seed,
        )

        results.append(model.generate(config))

    for i in range(1, len(results)):
        assert results[i] != results[i - 1]


def test_generate_same_seed():
    model, tokenizer = prepare_models()

    seed = 12345

    prompt = compose_prompt(
        prompt="1girl, cat ears",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=seed,
    )

    output1 = model.generate(config)
    output2 = model.generate(config)

    assert output1 == output2


def test_generate_different_temperature():
    model, tokenizer = prepare_models()

    temperatures = [0.5, 1.0, 1.5]
    results = []

    for temperature in temperatures:
        prompt = compose_prompt(
            prompt="1girl, cat ears",
        )
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=42,
            temperature=temperature,
        )

        results.append(model.generate(config))

    assert results[0] != results[1]
    assert results[1] != results[2]
    assert results[0] != results[2]


def test_generate_different_top_p():
    model, tokenizer = prepare_models()

    top_ps = [0.5, 0.9, 0.99]
    results = []

    for top_p in top_ps:
        prompt = compose_prompt(
            prompt="1girl, cat ears",
        )
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=42,
            top_p=top_p,
        )

        results.append(model.generate(config))

    assert results[0] != results[1]
    assert results[1] != results[2]
    assert results[0] != results[2]


def test_generate_different_top_k():
    model, tokenizer = prepare_models()

    top_ks = [50, 100, 200]
    results = []

    for top_k in top_ks:
        prompt = compose_prompt(
            prompt="1girl, cat ears",
        )
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=42,
            top_k=top_k,
        )

        results.append(model.generate(config))

    assert results[0] != results[1]
    assert results[1] != results[2]
    assert results[0] != results[2]


def test_generate_different_prompt():
    model, tokenizer = prepare_models()

    prompts = [
        compose_prompt(
            prompt="1girl, cat ears",
        ),
        compose_prompt(
            prompt="2girls",
        ),
        compose_prompt(
            prompt="no humans, scenery",
        ),
    ]

    results = []

    for prompt in prompts:
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=42,
        )

        results.append(model.generate(config))

    assert results[0] != results[1]
    assert results[1] != results[2]


def test_generate_with_ban_token_ids():
    model, tokenizer = prepare_models()

    prompt = compose_prompt(
        prompt="1girl, solo",
        length="long",
        identity="none",
        aspect_ratio="tall",
        rating="sfw",
    )

    ban_tags = "animal ear fluff, animal ears"
    ban_token_ids = tokenizer.encode(ban_tags)

    for index in range(0, 30):
        config = get_generation_config(
            prompt=prompt,
            tokenizer=tokenizer,
            seed=index,
            ban_token_ids=ban_token_ids,
            temperature=0.9,
            top_p=0.9,
            top_k=100,
        )

        result = model.generate(config)

        assert "animal ears" not in result, result


def test_generate_next_token():
    model, tokenizer = prepare_models()

    prompt = compose_prompt(
        prompt="1girl, cat ears",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=42,
    )

    tokens = tokenizer.encode(prompt)
    cache = GenerationCache(tokens)

    for _ in range(0, 10):
        token, cache = model._get_next_token(config, cache)

        assert token is not None
        assert token > 0


def test_generate_stream():
    model, tokenizer = prepare_models()

    prompt = compose_prompt(
        prompt="1girl, cat ears",
        length="long",
    )
    config = get_generation_config(
        prompt=prompt,
        tokenizer=tokenizer,
        seed=42,
    )

    for tag in model.generate_stream(config):
        assert tag is not None
        assert isinstance(tag, str)



from typing import Literal

from . import dartrs

DType = Literal["fp16", "fp32"]
Device = Literal["cpu", "cuda"]


def get_generation_config(
    prompt: str,
    tokenizer: dartrs.DartTokenizer,
    device: dartrs.DartDevice = dartrs.DartDevice.Cpu(),
    eos_token: str | None = None,
    max_new_tokens: int | None = 256,
    temperature: float | None = 1.0,
    top_p: float | None = 1.0,
    top_k: int | None = 100,
    ban_token_ids: list[int] | None = None,
    seed: int | None = None,
) -> dartrs.GenerationConfig:
    return dartrs.GenerationConfig(
        device=device,
        tokenizer=tokenizer,
        prompt=prompt,
        eos_token=eos_token,
        max_new_tokens=max_new_tokens,
        temperature=temperature,
        top_p=top_p,
        top_k=top_k,
        ban_token_ids=ban_token_ids,
        seed=seed,
    )



from dartrs.dartrs import DartTokenizer
from dartrs.utils import get_generation_config
from dartrs.v2 import (
    compose_prompt,
    MixtralModel,
)
import time

MODEL_NAME = "p1atdev/dart-v2-moe-sft"

model = MixtralModel.from_pretrained(MODEL_NAME, dtype="fp16")
tokenizer = DartTokenizer.from_pretrained(MODEL_NAME)

config = get_generation_config(
    prompt=compose_prompt(
        copyright="vocaloid",
        character="hatsune miku",
        rating="general",  # sfw, general, sensitive, nsfw, questionable, explicit
        aspect_ratio="tall",  # ultra_wide, wide, square, tall, ultra_tall
        length="medium",  # very_short, short, medium, long, very_long
        identity="none",  # none, lax, strict
        prompt="1girl, cat ears",
    ),
    tokenizer=tokenizer,
    seed=42,
)

start = time.time()
output = model.generate(config)
end = time.time()

print(output)
print(f"Time taken: {end - start:.2f}s")



from abc import ABC

class DartDType:
    BF16: ...
    FP16: ...
    FP32: ...
    def __init__(self, dtype: str) -> None: ...

class DartDevice:
    def __init__(self, device: str) -> None: ...
    @classmethod
    def Cpu(cls) -> ...: ...
    @classmethod
    def Cuda(cls, id: int) -> ...: ...

class GenerationConfig:
    def __init__(
        self,
        device: DartDevice,
        tokenizer: DartTokenizer,
        prompt: str,
        eos_token: str | None = None,
        max_new_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        top_k: int | None = None,
        ban_token_ids: list[int] | None = None,
        seed: int | None = None,
    ) -> None: ...
    def tokenizer(self) -> DartTokenizer: ...
    def prompt(self) -> str: ...
    def max_new_tokens(self) -> int: ...

class GenerationCache:
    def __init__(self, input_tokens: list[int]) -> None: ...
    def input_tokens(self) -> list[int]: ...
    def output_tokens(self) -> list[int]: ...
    def finished(self) -> bool: ...

class DartV2Mistral:
    def __init__(
        self,
        hub_name: str,
        revision: str | None = None,
        dtype: DartDType = DartDType.FP32,
        device: DartDevice = DartDevice.Cpu(),
        auth_token: str | None = None,
    ) -> None: ...
    def generate(self, config: GenerationConfig) -> str:
        raise NotImplementedError

    def get_next_token(
        self,
        config: GenerationConfig,
        cache: GenerationCache,
    ) -> tuple[int, GenerationCache]: ...
    def _clear_kv_cache(self) -> None: ...

class DartV2Mixtral:
    def __init__(
        self,
        hub_name: str,
        revision: str | None = None,
        dtype: DartDType = DartDType.FP32,
        device: DartDevice = DartDevice.Cpu(),
        auth_token: str | None = None,
    ) -> None: ...
    def generate(self, config: GenerationConfig) -> str:
        raise NotImplementedError

    def get_next_token(
        self,
        config: GenerationConfig,
        cache: GenerationCache,
    ) -> tuple[int, GenerationCache]: ...
    def _clear_kv_cache(self) -> None: ...

class DartTokenizer:
    @staticmethod
    def from_pretrained(
        identifier,
        revision: str | None = None,
        auth_token: str | None = None,
    ) -> DartTokenizer: ...
    def encode(self, text: str) -> list[int]: ...
    def decode(
        self, token_ids: list[int], skip_special_tokens: bool | None = None
    ) -> str:
        """Decodes tokens and returns a the concatenated text."""
        ...

    def decode_tags(
        self, token_ids: list[int], skip_special_tokens: bool | None = None
    ) -> list[str]:
        """Decodes tokens and returns a list of tags."""
        ...

    def tokenize(self, text: str) -> list[str]:
        """Tokenizes text and returns a list of tokens."""
        ...

    def get_vocab(self, with_added_tokens: bool | None = True) -> dict[str, int]:
        """Returns the vocabulary as a dictionary. The keys are the tokens and the values are the token IDs."""
        ...

    def get_added_tokens(self) -> list[str]:
        """Returns the added tokens."""
        ...

class Tag(ABC):
    def __init__(self, tag: str) -> None: ...
    def to_tag(self) -> str: ...

class LengthTag(Tag):
    VeryShort: LengthTag
    Short: LengthTag
    Medium: LengthTag
    Long: LengthTag
    VeryLong: LengthTag

class AspectRatioTag(Tag):
    UltraWide: AspectRatioTag
    Wide: AspectRatioTag
    Square: AspectRatioTag
    Tall: AspectRatioTag
    UltraTall: AspectRatioTag

class RatingTag(Tag):
    Sfw: RatingTag
    General: RatingTag
    Sensitive: RatingTag
    Nsfw: RatingTag
    Questionable: RatingTag
    Explicit: RatingTag

class IdentityTag(Tag):
    Free: IdentityTag
    Lax: IdentityTag
    Strict: IdentityTag

class SpecialTag(Tag):
    Bos: SpecialTag
    Eos: SpecialTag
    CopyrightStart: SpecialTag
    CopyrightEnd: SpecialTag
    CharacterStart: SpecialTag
    CharacterEnd: SpecialTag
    GeneralStart: SpecialTag
    GeneralEnd: SpecialTag
    InputEnd: SpecialTag

def compose_prompt_v2(
    copyright: str,
    character: str,
    rating: RatingTag,
    aspect_ratio: AspectRatioTag,
    length: LengthTag,
    identity_level: IdentityTag,
    prompt: str,
    do_completion: bool,
) -> str: ...



from dartrs.dartrs import DartTokenizer, GenerationConfig, DartDType
from dartrs.utils import get_generation_config
from dartrs.v2 import (
    compose_prompt,
    MixtralModel,
    V2Model,
)
import time
import os

MODEL_NAME = "p1atdev/dart-v2-moe-sft"


def prepare_models():
    model = MixtralModel.from_pretrained(MODEL_NAME, dtype="fp16")
    tokenizer = DartTokenizer.from_pretrained(MODEL_NAME)

    return model, tokenizer


def generate(model: V2Model, config: GenerationConfig):
    start = time.time()
    output = model.generate(config)
    end = time.time()

    print(output)
    print(f"Time taken: {end - start:.2f}s")


def generate_stream(model: V2Model, config: GenerationConfig):
    start = time.time()
    for tag in model.generate_stream(config):
        if tag.strip() == "":
            continue
        os.write(1, tag.encode("utf-8") + b", ")
    end = time.time()

    print()
    print(f"Time taken: {end - start:.2f}s")


def main():
    model, tokenizer = prepare_models()

    # generate 5 times
    generate(
        model,
        get_generation_config(
            prompt=compose_prompt(
                copyright="",
                character="",
                rating="general",
                aspect_ratio="tall",
                length="medium",
                identity="none",
                prompt="1girl, cat ears",
            ),
            tokenizer=tokenizer,
            seed=42,
        ),
    )
    generate(
        model,
        get_generation_config(
            prompt=compose_prompt(
                prompt="2girls",
            ),
            tokenizer=tokenizer,
            seed=999999,
        ),
    )

    generate_stream(
        model,
        get_generation_config(
            prompt=compose_prompt(
                prompt="1girl, solo",
                length="long",
            ),
            tokenizer=tokenizer,
        ),
    )


if __name__ == "__main__":
    main()
